@@ -17,22 +17,13 @@
 #ifndef _ETHERINFO_H
 #define _ETHERINFO_H
 
-#include <netlink/addr.h>
-#include <netlink/netlink.h>
-#include <netlink/handlers.h>
-#include <netlink/route/link.h>
-#include <netlink/route/addr.h>
-#include <arpa/inet.h>
+typedef enum {NLQRY_ADDR4, NLQRY_ADDR6} nlQuery; /**<  Supported query types in the etherinfo code */
 
-typedef enum {NLQRY_LINK, NLQRY_ADDR} nlQuery; /**<  Supported query types in the etherinfo code */
+int get_etherinfo_link(PyEtherInfo *data);
+PyObject * get_etherinfo_address(PyEtherInfo *self, nlQuery query);
 
-int get_etherinfo(struct etherinfo_obj_data *data, nlQuery query);
-void free_etherinfo(struct etherinfo *ptr);
-void dump_etherinfo(FILE *, struct etherinfo *);
-
-void free_ipv6addresses(struct ipv6address *ptr);
-
-int open_netlink(struct etherinfo_obj_data *);
-void close_netlink(struct etherinfo_obj_data *);
+int open_netlink(PyEtherInfo *);
+struct nl_sock * get_nlc();
+void close_netlink(PyEtherInfo *);
 
 #endif
