@@ -16,6 +16,7 @@
  * General Public License for more details.
  */
 #include <Python.h>
+#include <bytesobject.h>
 
 #include <errno.h>
 #include <stddef.h>
@@ -60,7 +61,7 @@ static PyObject *get_active_devices(PyObject *self __unused, PyObject *args __un
 
 	list = PyList_New(0);
 	for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
-		PyObject *str = PyString_FromString(ifa->ifa_name);
+		PyObject *str = PyBytes_FromString(ifa->ifa_name);
 		/* names are not unique (listed for both ipv4 and ipv6) */
 		if (!PySequence_Contains(list, str) && (ifa->ifa_flags & (IFF_UP))) {
 			PyList_Append(list, str);
@@ -104,7 +105,7 @@ static PyObject *get_devices(PyObject *self __unused, PyObject *args __unused)
 		while (*name == ' ')
 			name++; /* skip over leading whitespace if any */
 
-		str = PyString_FromString(name);
+		str = PyBytes_FromString(name);
 		PyList_Append(list, str);
 		Py_DECREF(str);
 	}
@@ -119,7 +120,7 @@ static PyObject *get_hwaddress(PyObject *self __unused, PyObject *args)
 	const char *devname;
 	char hwaddr[20];
 
-	if (!PyArg_ParseTuple(args, "s", &devname))
+	if (!PyArg_ParseTuple(args, "s*", &devname))
 		return NULL;
 
 	/* Setup our request structure. */
@@ -151,7 +152,7 @@ static PyObject *get_hwaddress(PyObject *self __unused, PyObject *args)
 		(unsigned int)ifr.ifr_hwaddr.sa_data[4] % 256,
 		(unsigned int)ifr.ifr_hwaddr.sa_data[5] % 256);
 
-	return PyString_FromString(hwaddr);
+	return PyBytes_FromString(hwaddr);
 }
 
 static PyObject *get_ipaddress(PyObject *self __unused, PyObject *args)
@@ -161,7 +162,7 @@ static PyObject *get_ipaddress(PyObject *self __unused, PyObject *args)
 	const char *devname;
 	char ipaddr[20];
 
-	if (!PyArg_ParseTuple(args, "s", &devname))
+	if (!PyArg_ParseTuple(args, "s*", &devname))
 		return NULL;
 
 	/* Setup our request structure. */
@@ -191,7 +192,7 @@ static PyObject *get_ipaddress(PyObject *self __unused, PyObject *args)
 		(unsigned int)ifr.ifr_addr.sa_data[4] % 256,
 		(unsigned int)ifr.ifr_addr.sa_data[5] % 256);
 
-	return PyString_FromString(ipaddr);
+	return PyBytes_FromString(ipaddr);
 }
 
 
@@ -218,10 +219,16 @@ static PyObject *get_interfaces_info(PyObject *self __unused, PyObject *args) {
 
 	/* Parse input arguments if we got them */
 	if( inargs != NULL ) {
-		if( PyString_Check(inargs) ) { /* Input argument is just a string */
+		if( PyBytes_Check(inargs) ) { /* Input argument is just a string */
 			fetch_devs_len = 1;
 			fetch_devs = calloc(1, sizeof(char *));
-			fetch_devs[0] = PyString_AsString(inargs);
+			fetch_devs[0] = PyBytes_AsString(inargs);
+#if PY_MAJOR_VERSION >= 3
+		} else if( PyUnicode_Check(inargs) ) { /* Input argument is just a ustring */
+			fetch_devs_len = 1;
+			fetch_devs = calloc(1, sizeof(char *));
+			fetch_devs[0] = PyUnicode_AsUTF8(inargs);
+#endif
 		} else if( PyTuple_Check(inargs) ) { /* Input argument is a tuple list with devices */
 			int j = 0;
 
@@ -229,8 +236,8 @@ static PyObject *get_interfaces_info(PyObject *self __unused, PyObject *args) {
 			fetch_devs = calloc(fetch_devs_len+1, sizeof(char *));
 			for( i = 0; i < fetch_devs_len; i++ ) {
 				PyObject *elmt = PyTuple_GetItem(inargs, i);
-				if( elmt && PyString_Check(elmt) ) {
-					fetch_devs[j++] = PyString_AsString(elmt);
+				if( elmt && PyBytes_Check(elmt) ) {
+					fetch_devs[j++] = PyBytes_AsString(elmt);
 				}
 			}
 			fetch_devs_len = j;
@@ -241,8 +248,8 @@ static PyObject *get_interfaces_info(PyObject *self __unused, PyObject *args) {
 			fetch_devs = calloc(fetch_devs_len+1, sizeof(char *));
 			for( i = 0; i < fetch_devs_len; i++ ) {
 				PyObject *elmt = PyList_GetItem(inargs, i);
-				if( elmt && PyString_Check(elmt) ) {
-					fetch_devs[j++] = PyString_AsString(elmt);
+				if( elmt && PyBytes_Check(elmt) ) {
+					fetch_devs[j++] = PyBytes_AsString(elmt);
 				}
 			}
 			fetch_devs_len = j;
@@ -267,7 +274,7 @@ static PyObject *get_interfaces_info(PyObject *self __unused, PyObject *args) {
 			free(fetch_devs);
 			return NULL;
                 }
-		dev->device = PyString_FromString(fetch_devs[i]);
+		dev->device = PyBytes_FromString(fetch_devs[i]);
 		dev->hwaddress = NULL;
 		dev->index = -1;
 
@@ -287,7 +294,7 @@ static PyObject *get_flags (PyObject *self __unused, PyObject *args)
 	const char *devname;
 	int fd, err;
 
-	if (!PyArg_ParseTuple(args, "s", &devname))
+	if (!PyArg_ParseTuple(args, "s*", &devname))
 		return NULL;
 
 	/* Setup our request structure. */
@@ -320,7 +327,7 @@ static PyObject *get_netmask (PyObject *self __unused, PyObject *args)
 	const char *devname;
 	char netmask[20];
 
-	if (!PyArg_ParseTuple(args, "s", &devname))
+	if (!PyArg_ParseTuple(args, "s*", &devname))
 		return NULL;
 
 	/* Setup our request structure. */
@@ -350,7 +357,7 @@ static PyObject *get_netmask (PyObject *self __unused, PyObject *args)
 		(unsigned int)ifr.ifr_netmask.sa_data[4] % 256,
 		(unsigned int)ifr.ifr_netmask.sa_data[5] % 256);
 
-	return PyString_FromString(netmask);
+	return PyBytes_FromString(netmask);
 }
 
 static PyObject *get_broadcast(PyObject *self __unused, PyObject *args)
@@ -360,7 +367,7 @@ static PyObject *get_broadcast(PyObject *self __unused, PyObject *args)
 	const char *devname;
 	char broadcast[20];
 
-	if (!PyArg_ParseTuple(args, "s", &devname))
+	if (!PyArg_ParseTuple(args, "s*", &devname))
 		return NULL;
 
 	/* Setup our request structure. */
@@ -390,7 +397,7 @@ static PyObject *get_broadcast(PyObject *self __unused, PyObject *args)
 		(unsigned int)ifr.ifr_broadaddr.sa_data[4] % 256,
 		(unsigned int)ifr.ifr_broadaddr.sa_data[5] % 256);
 
-	return PyString_FromString(broadcast);
+	return PyBytes_FromString(broadcast);
 }
 
 static PyObject *get_module(PyObject *self __unused, PyObject *args)
@@ -401,7 +408,7 @@ static PyObject *get_module(PyObject *self __unused, PyObject *args)
 	char buf[2048];
 	const char *devname;
 
-	if (!PyArg_ParseTuple(args, "s", &devname))
+	if (!PyArg_ParseTuple(args, "s*", &devname))
 		return NULL;
 
 	/* Setup our control structures. */
@@ -455,12 +462,12 @@ static PyObject *get_module(PyObject *self __unused, PyObject *args)
 			return NULL;
 		} else {
 			PyErr_Clear();
-			return PyString_FromString(driver);
+			return PyBytes_FromString(driver);
 		}
 	}
 
 	close(fd);
-	return PyString_FromString(((struct ethtool_drvinfo *)buf)->driver);
+	return PyBytes_FromString(((struct ethtool_drvinfo *)buf)->driver);
 }
 
 static PyObject *get_businfo(PyObject *self __unused, PyObject *args)
@@ -471,7 +478,7 @@ static PyObject *get_businfo(PyObject *self __unused, PyObject *args)
 	char buf[1024];
 	const char *devname;
 
-	if (!PyArg_ParseTuple(args, "s", &devname))
+	if (!PyArg_ParseTuple(args, "s*", &devname))
 		return NULL;
 
 	/* Setup our control structures. */
@@ -499,7 +506,7 @@ static PyObject *get_businfo(PyObject *self __unused, PyObject *args)
 	}
 
 	close(fd);
-	return PyString_FromString(((struct ethtool_drvinfo *)buf)->bus_info);
+	return PyBytes_FromString(((struct ethtool_drvinfo *)buf)->bus_info);
 }
 
 static int send_command(int cmd, const char *devname, void *value)
@@ -537,7 +544,7 @@ static int get_dev_value(int cmd, PyObject *args, void *value)
 	const char *devname;
 	int err = -1;
 
-	if (PyArg_ParseTuple(args, "s", &devname))
+	if (PyArg_ParseTuple(args, "s*", &devname))
 		err = send_command(cmd, devname, value);
 
 	return err;
@@ -559,7 +566,7 @@ static int dev_set_int_value(int cmd, PyObject *args)
 	struct ethtool_value eval;
 	const char *devname;
 
-	if (!PyArg_ParseTuple(args, "si", &devname, &eval.data))
+	if (!PyArg_ParseTuple(args, "s*i", &devname, &eval.data))
 		return -1;
 
 	return send_command(cmd, devname, &eval);
@@ -666,7 +673,7 @@ static PyObject *__struct_desc_create_dict(struct struct_desc *table,
 
 		switch (d->size) {
 		case sizeof(uint32_t):
-			objval = PyInt_FromLong(*(uint32_t *)val);
+			objval = PyLong_FromLong(*(uint32_t *)val);
 			break;
 		}
 
@@ -711,7 +718,7 @@ static int __struct_desc_from_dict(struct struct_desc *table,
 				PyErr_SetString(PyExc_IOError, buf);
 				return -1;
 			}
-			*(uint32_t *)val = PyInt_AsLong(obj);
+			*(uint32_t *)val = PyLong_AsLong(obj);
 			break;
 		default:
 			snprintf(buf, sizeof(buf),
@@ -744,7 +751,7 @@ static PyObject *set_coalesce(PyObject *self __unused, PyObject *args)
 	const char *devname;
 	PyObject *dict;
 
-	if (!PyArg_ParseTuple(args, "sO", &devname, &dict))
+	if (!PyArg_ParseTuple(args, "s*O", &devname, &dict))
 		return NULL;
 
 	if (struct_desc_from_dict(ethtool_coalesce_desc, &coal, dict) != 0)
@@ -784,7 +791,7 @@ static PyObject *set_ringparam(PyObject *self __unused, PyObject *args)
 	const char *devname;
 	PyObject *dict;
 
-	if (!PyArg_ParseTuple(args, "sO", &devname, &dict))
+	if (!PyArg_ParseTuple(args, "s*O", &devname, &dict))
 		return NULL;
 
 	if (struct_desc_from_dict(ethtool_ringparam_desc, &ring, dict) != 0)
@@ -898,19 +905,34 @@ static struct PyMethodDef PyEthModuleMethods[] = {
 	{	.ml_name = NULL, },
 };
 
+#if PY_MAJOR_VERSION >= 3
+  #define MOD_ERROR_VAL NULL
+  #define MOD_SUCCESS_VAL(val) val
+  #define MOD_INIT(name) PyMODINIT_FUNC PyInit_##name(void)
+  #define MOD_DEF(ob, name, doc, methods) \
+          static struct PyModuleDef moduledef = { \
+            PyModuleDef_HEAD_INIT, name, doc, -1, methods, }; \
+          ob = PyModule_Create(&moduledef);
+#else
+  #define MOD_ERROR_VAL
+  #define MOD_SUCCESS_VAL(val)
+  #define MOD_INIT(name) void init##name(void)
+  #define MOD_DEF(ob, name, doc, methods) \
+          ob = Py_InitModule3(name, methods, doc);
+#endif
 
-PyMODINIT_FUNC initethtool(void)
+MOD_INIT(ethtool)
 {
 	PyObject *m;
-	m = Py_InitModule3("ethtool", PyEthModuleMethods, "Python ethtool module");
+	MOD_DEF(m, "ethtool", "Python ethtool module", PyEthModuleMethods);
 
 	// Prepare the ethtool.etherinfo class
 	if (PyType_Ready(&PyEtherInfo_Type) < 0)
-		return;
+		return MOD_ERROR_VAL;
 
 	// Prepare the ethtool IPv6 and IPv4 address types
 	if (PyType_Ready(&ethtool_netlink_ip_address_Type))
-		return;
+		return MOD_ERROR_VAL;
 
 	// Setup constants
 	PyModule_AddIntConstant(m, "IFF_UP", IFF_UP);			/* Interface is up. */
@@ -932,6 +954,8 @@ PyMODINIT_FUNC initethtool(void)
 	PyModule_AddIntConstant(m, "AF_INET", AF_INET);                 /* IPv4 interface */
 	PyModule_AddIntConstant(m, "AF_INET6", AF_INET6);               /* IPv6 interface */
 	PyModule_AddStringConstant(m, "version", "python-ethtool v" VERSION);
+
+	return MOD_SUCCESS_VAL(m);
 }
 
 
